<?php

namespace Drupal\Tests\media_twig_tools\Unit\Base;

use Drupal\Tests\UnitTestCase;
use Drupal\file\Entity\File;
use Drupal\file\FileInterface;
use Drupal\image\Entity\ImageStyle;
use Drupal\image\ImageStyleStorage;
use Drupal\media_twig_tools\Config\MediaTwigToolsConfig;
use Drupal\media_twig_tools\Services\ImageBuilder;

/**
 * Abstract tools for unit tests.
 */
abstract class AbstractUnitTestCase extends UnitTestCase {

  /**
   * ImageBuilder.
   *
   * @var \Drupal\media_twig_tools\Services\ImageBuilder
   */
  protected ImageBuilder $imageBuilders;

  /**
   * Tests data.
   *
   * @var array
   */
  protected array $testData = [];

  /**
   * Input File.
   *
   * @var \Drupal\file\FileInterface
   */
  protected FileInterface $file;

  /**
   * Return mock file data.
   *
   * @param string $name
   *   The directory name.
   *
   * @return array
   *   The files.
   */
  protected function getFiles(string $name): array {
    $path = __DIR__ . '/../../data/' . $name . '/*.json';

    $files = [];

    foreach (glob($path) as $path) {
      $id = str_replace('.json', '', basename($path));
      $scopes = array_slice(explode('/', dirname($path)), 14);

      $files[$id] = [json_decode(file_get_contents($path), TRUE)];
      $files[$id]['scopes'] = $scopes;
    }

    return $files;
  }

  /**
   * Return the config mock.
   *
   * @return \Drupal\media_twig_tools\Config\MediaTwigToolsConfig|\PHPUnit\Framework\MockObject\MockObject
   *   The mock.
   */
  protected function mockConfig() {
    $conf = $this->createMock(MediaTwigToolsConfig::class);

    $conf->expects($this->any())
      ->method('getDirname')
      ->willReturn('builder');

    $conf->expects($this->any())
      ->method('getNoStyle')
      ->willReturn('landscape');

    // Init the image style storage mock.
    $imageStyleStorage = $this->createMock(ImageStyleStorage::class);
    $imageStyleStorage->expects($this->any())
      ->method('load')
      ->willReturnCallback(function ($id) {
        $imageStyleMock = $this->createMock(ImageStyle::class);
        $imageStyleMock->expects($this->any())
          ->method('id')
          ->willReturnCallback(function () use ($id) {
            return $id;
          });

        return $imageStyleMock;
      });

    return $conf;
  }

  /**
   * Mock file.
   *
   * @return \Drupal\file\Entity\File|\PHPUnit\Framework\MockObject\MockObject
   *   The file mock.
   */
  protected function mockFile() {
    // Define file.
    $file = $this->createMock(File::class);
    $file->expects($this->any())
      ->method('id')
      ->willReturn(1);
    $file->expects($this->any())
      ->method('getFileUri')
      ->willReturn('/sites/default/files/test.png');
    return $file;
  }

}
